#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const program = require("commander");
const path_1 = require("path");
const fs_extra_1 = require("fs-extra");
const util_1 = require("util");
const child_process_1 = require("child_process");
// no types from `download-git-repo`
const download = util_1.promisify(require('download-git-repo'));
const execAsync = util_1.promisify(child_process_1.exec);
const templateMap = {
    datasource: {
        projectName: 'imgcook/pipcook-datasource-template',
        defaultBranch: 'main',
        url: 'https://github.com/imgcook/pipcook-datasource-template'
    },
    dataflow: {
        projectName: 'imgcook/pipcook-dataflow-template',
        defaultBranch: 'main',
        url: 'https://github.com/imgcook/pipcook-dataflow-template'
    },
    model: {
        projectName: 'imgcook/pipcook-model-template',
        defaultBranch: 'main',
        url: 'https://github.com/imgcook/pipcook-model-template'
    }
};
function createScript(scriptType, scriptName, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const projectName = path_1.basename(scriptName);
            const destDir = path_1.resolve(scriptName);
            const pkgFile = path_1.join(destDir, 'package.json');
            if (yield fs_extra_1.pathExists(destDir)) {
                throw new TypeError(`The directory ${scriptName} already exists`);
            }
            const branch = opts.type ? opts.type : templateMap[scriptType].defaultBranch;
            const template = `${templateMap[scriptType].projectName}#${branch}`;
            yield download(template, destDir, { clone: true });
            if (yield fs_extra_1.pathExists(pkgFile)) {
                const pkg = yield fs_extra_1.readJson(pkgFile);
                pkg.name = projectName;
                yield fs_extra_1.writeJson(pkgFile, pkg, { spaces: 2 });
                console.log('initializing project');
                yield execAsync('npm i', { cwd: destDir });
            }
            console.info('initialize script project successfully');
        }
        catch (err) {
            console.error(`create from template failed: ${err.message}`);
            process.exit(1);
        }
    });
}
;
(function () {
    return __awaiter(this, void 0, void 0, function* () {
        for (const key in templateMap) {
            program
                .command(`${key} <scriptName>`)
                .option('-t --type <templeteType>')
                .description(`create pipcook ${key} script from template. Find templetes here: ${templateMap[key].url}`)
                .action((scriptName, opts) => createScript(key, scriptName, opts));
        }
        program.parse(process.argv);
    });
})();
//# sourceMappingURL=pipcook-script.js.map